
/*******************************************************************************

Author : Mauro Laurenti
Version : 1.1
Created on Date : 4/9/2006
Last update     : 25/01/2013

CopyRight 2006-2013 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This libary supports the Real Time Clock Calendar PCF8563 or compatible
devices

******************************************************************************/



#ifndef PCF8563_H
#define PCF8563_H

#ifdef __XC8
    #include <xc.h>
        #ifndef _PIC18
        #error "The PCF8563 Library supports only PIC18 devices"
    #endif
#endif

#ifdef __18CXX
        #include <i2c.h>
#endif


//**************************************************
// you must change this address if
// you change the device.
//**************************************************

#define WRITE_ADD 0xA2	

//**************************************************
// Definition of Day constants (Italian, English)
//**************************************************

// Italian
#define	DO 0x00	
#define	LU 0x01
#define	MA 0x02
#define	ME 0x03
#define	GI 0x04
#define	VE 0x05
#define	SA 0x06

// English
#define	SU 0x00
#define	MO 0x01 
#define	TU 0x02 
#define	WE 0x03 
#define	TR 0x04 
#define	FR 0x05 
#define	SA 0x06



//**************************************************
//        		Standard Constant
// enable value for the allarm bit (day,month,dayweek)
//**************************************************

#define ENABLE_ON  0b00000000
#define ENABLE_OFF 0b10000000


//**************************************************
//           Address constants
//**************************************************

#define CONTROL_REG_2_ADDR		0x01
#define SECONDS_ADDR 			0x02
#define MINUTES_ADDR 			0x03
#define HOURS_ADDR 			0x04
#define DAYS_ADDR 			0x05
#define DAY_WEEK_ADDR 			0x06
#define MONTHS_ADDR	 		0x07
#define YEARS_ADDR	 		0x08
#define MINUTS_ALARM_ADDR	 	0x09
#define HOURS_ALARM_ADDR	 	0x0A
#define DAYS_ALARM_ADDR	 		0x0B
#define DAY_WEEK_ALARM_ADDR             0x0C


//**************************************************
//           Limits constants
//**************************************************
#define MAX_MINUTES  0x60
#define MAX_HOURS 	 0x24
#define MAX_YEARS 	 0xA0
#define MAX_MONTHS	 0x13
#define MAX_DAYS 	 0x32


/**
 * This function writes the seconds in the RTC.
 *
 * @param seconds Seconds to be written, as 2 digits BCD format (eg. 22 sec. is 0x22 or 0b00100010).
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @warning Seconds are in BCD format. Decimal format won't work!
 */
signed char set_seconds_RTCC (unsigned char seconds);


/**
 * This function reads the second from the RTC.
 *
 * @return Second that are read back, in BCD format (eg. 22 sec. is 0x22 or 0b00100010).
 *
 * @note Seconds are in BCD format.
 */
unsigned char get_seconds_RTCC (void);


/**
 * This function writes the seconds in the RTC.
 *
 * @param minutes Minutes to be written, as 2 digits BCD format (eg. 22 min. is 0x22 or 0b00100010).
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @warning Minutes are in BCD format. Decimal format won't work!
 */
signed char set_minutes_RTCC (unsigned char minutes);


/**
 * This function reads the muntes from the RTC.
 *
 * @return Minutes that are read back, in BCD format (eg. 22 min. is 0x22 or 0b00100010).
 *
 * @note Seconds are in BCD format.
 */
unsigned char get_minutes_RTCC (void);


/**
 * This function writes the hours in the RTC.
 *
 * @param hours Hours to be written, as 2 digits BCD format (eg. 12 is 0x12 or 0b00010010).
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @warning Hours are in BCD format. Decimal format won't work!
 */
signed char set_hours_RTCC (unsigned char hours);


/**
 * This function reads the hours from the RTC.
 *
 * @return Hours that are read back, in BCD format (eg. 12 is 0x12 or 0b00010010).
 *
 * @note Hours are in BCD format.
 */
unsigned char get_hours_RTCC (void);


/**
 * This function reads the time from the RTC. The format is HH:MM.ss
 *
 * @return Time that is read back, as ASCII string termianted with /0
 *
 */
unsigned char* get_time_seconds_RTCC (void);


/**
 * This function reads the time from the RTC. The format is HH:MM (without seconds)
 *
 * @return Time that is read back, as ASCII string termianted with /0
 *
 */
unsigned char* get_time_RTCC (void);


/**
 * This function writes the days in the RTC.
 *
 * @param days Day to be written, as 2 digits BCD format (eg. 12 is 0x12 or 0b00010010).
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @warning Days are in BCD format. Decimal format won't work!
 */
signed char set_days_RTCC (unsigned char days);


/**
 * This function reads the day from the RTC.
 *
 * @return Days that are read back, in BCD format (eg. 12 is 0x12 or 0b00010010).
 *
 * @note Days are in BCD format.
 */
unsigned char get_days_RTCC (void);


/**
 * This function writes the WeekDay in the RTC.
 *
 * @param day_of_the_week Day of the week to be written [0..6].
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @note Use the constants to specify the Day of the week (either Italian or English).
 *       [DO, LU, MA, ME, GI, VE, SA] or [SU, MO, TU, WE, TR, FR, SA]
 */
signed char set_day_of_the_week_RTCC (unsigned char day_of_the_week);


/**
 * This function reads the Weekday from the RTC.
 *
 * @return Days of the week that is read back,.
 *
 * @note Use the constants to compare the Day of the week (either Italian or English).
 *       [DO, LU, MA, ME, GI, VE, SA] or [SU, MO, TU, WE, TR, FR, SA]
 */
signed char get_day_of_the_week_RTCC (void);


/**
 * This function writes the moths in the RTC.
 *
 * @param months Months to be written, as 2 digits BCD format (eg. 12 is 0x12 or 0b00010010).
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @warning Months are in BCD format. Decimal format won't work!
 */
signed char set_months_RTCC (unsigned char months);


/**
 * This function reads the months from the RTC.
 *
 * @return Months that are read back, in BCD format (eg. 12 is 0x12 or 0b00010010).
 *
 * @note Months are in BCD format.
 */
unsigned char get_months_RTCC (void);


/**
 * This function writes the Year in the RTC.
 *
 * @param years Year to be written, as 2 digits BCD format (eg. 2012 is only 0x12 or 0b00010010).
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @warning Years are in BCD format. Decimal format won't work!
 * @note      2012 is written as 12.
 */
signed char set_years_RTCC (unsigned char years);


/**
 * This function reads the Year from the RTC.
 *
 * @return Year that is read back, in BCD format (eg. 12 is 0x12 or 0b00010010 represents 2012).
 *
 * @note Year are in BCD format. 2012 is read as 0x12.
 */
unsigned char get_years_RTCC (void);



/**
 * This function reads the Date from the RTC. The Format DD/MM/YY
 *
 * @return Date that is read back, as ASCII string termianted with /0
 *
 */
unsigned char* get_date_RTCC (void);


/**
 * This function writes the minutes for the alarm, in the RTC.
 *
 * @param minutes Minutes to be written, as 2 digits BCD format (eg. 12 is 0x12 or 0b00010010).
 *
 * @param alarmEnable Enable the alarm or not (for matching) [Enable_ON , Enable_OFF]
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @warning Minutes are in BCD format. Decimal format won't work!
 *
 * @note Use the constants Enable_ON and Enable_OFF
 */
signed char set_minutes_alarm_RTCC (unsigned char minutes, unsigned char alarm_enable);


/**
 * This function writes the hours for the alarm, in the RTC.
 *
 * @param hours Hours to be written, as 2 digits BCD format (eg. 12 is 0x12 or 0b00010010).
 *
 * @param alarmEnable Enable the alarm or not (for matching) [Enable_ON , Enable_OFF]
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @warning Hours are in BCD format. Decimal format won't work!
 *
 * @note Use the constants Enable_ON and Enable_OFF
 */
signed char set_hours_alarm_RTCC (unsigned char hours, unsigned char alarm_enable);


/**
 * This function writes the Days for the alarm, in the RTC.
 *
 * @param days Days to be written, as 2 digits BCD format (eg. 12 is 0x12 or 0b00010010).
 *
 * @param alarmEnable Enable the alarm or not (for matching) [Enable_ON , Enable_OFF]
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @warning Days are in BCD format. Decimal format won't work!
 *
 * @note Use the constants Enable_ON and Enable_OFF
 */
signed char set_days_alarm_RTCC (unsigned char days, unsigned char alarm_enable);


/**
 * This function writes the WeekDays for the alarm, in the RTC.
 *
 * @param day_of_the_week Days of the week to be written.
 *
 * @param alarmEnable Enable the alarm or not (for matching) [Enable_ON , Enable_OFF]
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @note Use the constants to set the Day of the week (either Italian or English).
 *       [DO, LU, MA, ME, GI, VE, SA] or [SU, MO, TU, WE, TR, FR, SA]
 *
 * @note Use the constants Enable_ON and Enable_OFF
 */
signed char set_day_of_the_week_alarm_RTCC (unsigned char day_of_the_week, unsigned char alarm_enable);


/**
 * This function enables the interrupt pin funtionality.
 *
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 */
signed char enable_alarm_interrupt_RTCC (void);


/**
 * This function disable the interrupt pin funtionality.
 *
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 */
signed char disable_alarm_interrupt_RTCC (void);


/**
 * This function checks if the Alarm id ON using polling method.
 *
 *
 * @return status   Alarm status 1: ON  0:OFF
 *
 * @note If the alarm is on, AF flag is automatically cleaned.
 */
unsigned char is_alarm_ON_RTCC (void);


/**
 * This function increments the minutes inside the RTCC.
 *
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @note The function automatically checks the overflow and automatically incremetns the hours.
 */
signed char increment_minutes_RTCC (void);


/**
 * This function increments the hours inside the RTCC.
 *
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @note The function automatically checks the overflow.
 */
signed char increment_hours_RTCC (void);


/**
 * This function increments the Years inside the RTCC.
 *
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @note The function automatically checks the overflow.
 */
signed char increment_years_RTCC (void);


/**
 * This function increments the Months inside the RTCC.
 *
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @note The function automatically checks the overflow.
 */
signed char increment_months_RTCC (void);


/**
 * This function increments the Days inside the RTCC.
 *
 *
 * @return status   1: The info has been properly written
 *                 -1: Bus Collision error
 *                 -2: Not Ack error condition
 *                 -3: Write collision
 *
 * @note The function automatically checks the overflow.
 */

signed char increment_days_RTCC (void);


#endif

